//---------------------------------------------------------------------------
//
// TDirectDraw v1.0 - part of the "TDirectX Library" for Borland's C++ Builder
// ----------------
//
// (c) 1997,1998 by Darren J. Dwyer - All Rights Reserved
//
//
// Disclaimer:
//
// All source code, object files, documentation files, etc. included as
// part this package are supplied subject to the rules in the attached file
// LICENCE.TXT
//
// Please read the file LICENCE.TXT now.
//
//---------------------------------------------------------------------------
#ifndef TDirectDrawH
#define TDirectDrawH
//---------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
//---------------------------------------------------------------------------
#include <d3drmwin.h>
#include <direct.h>
//---------------------------------------------------------------------------
class TDirectDraw : public TComponent   // note: this object is only a TComponent
                                        // so that we have access to the Owner
                                        // property. !!DO NOT USE THIS OBJECT
                                        // AS A NORMAL VCL COMPONENT!!
{
    //----------------------------------------------------------------------------------
	// General Notes:
    //
    // (1)	This object "TDirectDraw" is designed primarily to be used with the
    // 		"TDirectDrawForm" object written by the same author. If this object
    //		is used separately, note that you should create only ONE (1) instance
    //		of this object. Users of the "TDirectDrawForm" object should never
    //		create an instance of this object as this is all handled within the
    //		"TDirectDrawForm" object.
    //
    // (2)	DirectDraw consists of a number of drawing surfaces that can be swapped,
    // 		drawn on and otherwise manipulated. Of these surfaces, the currently
    //		visible surface is called the primary surface. Attached to this surface
    //		are one or more hidden surfaces, stored in a linked-list arrangement, the
    //		first being called the back buffer. Only the primary surface and the back
    //      buffer can be manipulated at any one time, although they can be rotated
    //		so that you can draw on any or all of the available surfaces. See: Flip()
    //
    //----------------------------------------------------------------------------------

public:

    //----------------------------------------------------------------------------------
    // Func:	BeginPrimarySurfaceDrawing()
    // Desc:	This function creates a TCanvas that enables the use of standard VCL
    //			graphics functions to draw directly onto the Primary Surface (the current
    //			visual display).
    // Params:  left, top, right, bottom - the screen coordinates used for drawing
    // Returns:	a newly created TCanvas iff possible, NULL if something goes wrong.
    // Notes:	if used, EndPrimarySurfaceDrawing() must be called to release resources.
    //
    // Example:	void __fastcall Form1::Timer1Timer( TObject* Sender )
    //			{
    //			// standard test for DirectDraw availability
    //			if (DirectDraw == NULL) return;
    //			// get a canvas for the entire form area, directly manipulating the display
    //			TCanvas* canvas = DirectDraw->BeginPrimarySurfaceDrawing( 0,0,Width,Height );
    //			// if we have a valid drawing surface
    //			if (canvas != NULL)
    //				{
    //				// draw whatever
    //				canvas->Pen->Color = clRed;
    //				canvas->Brush->Color = clGreen;
    //				canvas->Rectangle( 0,0,Width,Height );
    //              }
    //			// finished drawing, release resources
    //			DirectDraw->EndPrimarySurfaceDrawing();
    //          }
    //----------------------------------------------------------------------------------
    TCanvas*	__fastcall	BeginPrimarySurfaceDrawing(int left, int top, int right, int bottom);

    //----------------------------------------------------------------------------------
    // Func:	EndPrimarySurfaceDrawing()
    // Desc:    This function is called to release resources used when calling the
    //			BeginPrimarySurfaceDrawing() function.
    //----------------------------------------------------------------------------------
	void		__fastcall	EndPrimarySurfaceDrawing();

    //----------------------------------------------------------------------------------
    // Func:	BeginBackBufferDrawing()
    // Desc:	This function creates a TCanvas that enables the use of standard VCL
    //			graphics functions to draw directly onto a Back Buffer (a non-visual
    //			display area).
    // Params:  left, top, right, bottom - the screen coordinates used for drawing
    //			autoflip - if true, region will be displayed upon calling
    //			EndBackBufferDrawing(). if false, region will not be automatically displayed.
    // Returns:	a newly created TCanvas iff possible, NULL if something goes wrong.
    // Notes:	if used, EndBackBufferDrawing() must be called to release resources.
    //
    // Example:	void __fastcall Form1::Timer1Timer( TObject* Sender )
    //			{
    //			// standard test for DirectDraw availability
    //			if (DirectDraw == NULL) return;
    //			// get an offscreen (back buffer) draw surface for the entire form area
    //			// and let it auto-flip when drawing is finished.
    //			TCanvas* canvas = DirectDraw->BeginBackBufferDrawing( 0,0,Width,Height, true );
    //			// if we have a valid drawing surface
    //			if (canvas != NULL)
    //				{
    //				// draw whatever
    //				canvas->Pen->Color = clRed;
    //				canvas->Brush->Color = clGreen;
    //				canvas->Rectangle( 0,0,Width,Height );
    //              }
    //			// finished drawing, release resources & auto-display the finished area
    //			DirectDraw->EndBackBufferDrawing();
    //          }
    //----------------------------------------------------------------------------------
	TCanvas*	__fastcall	BeginBackBufferDrawing(int left, int top, int right, int bottom, bool autoflip = false);

    //----------------------------------------------------------------------------------
    // Func:	EndBackBufferDrawing()
    // Desc:    This function is called to release resources used when calling the
    //			BeginBackBufferDrawing() function.
    //----------------------------------------------------------------------------------
    void		__fastcall	EndBackBufferDrawing();

    //----------------------------------------------------------------------------------
    // Func:	FlipToGDI()
    // Desc:	This function will continue to flip surfaces until the surface the GDI
    //			uses becomes the primary surface.
    // Returns:	Upon success, this function returns TRUE, otherwise it returns FALSE
    //----------------------------------------------------------------------------------
    bool		__fastcall	FlipToGDI();

    //----------------------------------------------------------------------------------
    // Func:	Flip()
    // Desc:    This function will rotate all current DirectDraw surfaces so that the
    //			back buffer will become the primary surface, the next back buffer will
    //			become the back buffer (where there are multiple back buffers) and the
    //			primary surface will become the last back buffer.
    // Params:	wait - 	if TRUE, this function will wait until the surfaces are
    //					available before flipping. if FALSE, the flip may not occur.
    // Returns:	Upon success, this function returns TRUE, otherwise it returns FALSE
    //----------------------------------------------------------------------------------
	bool		__fastcall	Flip(bool wait = true);

    //----------------------------------------------------------------------------------
    // Func:	FlipPrimarySurface()
    // Desc:    This function will copy the contents of the primary surface to the
    //			back buffer using accelerated functions if available.
    // Params:	wait - 	if TRUE, this function will wait until the surfaces are
    //					available before flipping. if FALSE, the flip may not occur.
    // Returns:	Upon success, this function returns TRUE, otherwise it returns FALSE
    //----------------------------------------------------------------------------------
    bool		__fastcall	FlipPrimarySurface(bool wait = true);

    //----------------------------------------------------------------------------------
    // Func:	FlipBackBuffer()
    // Desc:    This function will copy the contents of the back buffer to the
    //			primary surface using accelerated functions if available.
    // Params:	wait - 	if TRUE, this function will wait until the surfaces are
    //					available before flipping. if FALSE, the flip may not occur.
    // Returns:	Upon success, this function returns TRUE, otherwise it returns FALSE
    //----------------------------------------------------------------------------------
    bool		__fastcall	FlipBackBuffer(bool wait = true);

    //----------------------------------------------------------------------------------
    // Func:	FlipPrimarySurfaceRect()
    // Desc:    This function will copy a rectangular section of the primary surface to
    //			the back buffer using accelerated functions if available.
    // Params:	left,top,right,bottom -	the coords of the rectangle to copy
    //			wait - 	if TRUE, this function will wait until the surfaces are
    //					available before flipping. if FALSE, the flip may not occur.
    // Returns:	Upon success, this function returns TRUE, otherwise it returns FALSE
    //----------------------------------------------------------------------------------
    bool		__fastcall	FlipPrimarySurfaceRect(int left,int top,int right,int bottom, bool wait = true);

    //----------------------------------------------------------------------------------
    // Func:	FlipBackBufferRect()
    // Desc:    This function will copy a rectangular section of the back buffer to
    //			the primary surface using accelerated functions if available.
    // Params:	left,top,right,bottom -	the coords of the rectangle to copy
    //        	wait - 	if TRUE, this function will wait until the surfaces are
    //					available before flipping. if FALSE, the flip may not occur.
    // Returns:	Upon success, this function returns TRUE, otherwise it returns FALSE
    //----------------------------------------------------------------------------------
    bool		__fastcall	FlipBackBufferRect(int left,int top,int right,int bottom, bool wait = true);

    //----------------------------------------------------------------------------------
	// Func:	SetDisplayMode()
    // Desc:	This function will attempt to set the current display mode to the
    //			resolution specified.
    // Returns:	Upon success, this function returns TRUE, otherwise it returns FALSE
    //----------------------------------------------------------------------------------
	bool				__fastcall	SetDisplayMode(int width,int height,int bitsperpixel);

    //----------------------------------------------------------------------------------
	// Func:	RestoreDisplayMode()
    // Desc:	This function will attempt to restore the display mode to the resolution
    //			that was selected when the application started.
    // Returns:	Upon success, this function returns TRUE, otherwise it returns FALSE
    //----------------------------------------------------------------------------------
    bool				__fastcall	RestoreDisplayMode();

    //----------------------------------------------------------------------------------
	// Func:	RestoreLastDisplayMode()
    // Desc:	This function restore the display mode to the last display mode set
    //			using SetDisplayMode()
    // Returns:	Upon success, this function returns TRUE, otherwise it returns FALSE
    //----------------------------------------------------------------------------------
    void                __fastcall  RestoreLastDisplayMode();

    //----------------------------------------------------------------------------------
	// Func:	SetExclusiveMode()
    // Desc:	This function will switch DirectDraw to exclusive or full-screen mode
    // Returns:	Upon success, this function returns TRUE, otherwise it returns FALSE
    //----------------------------------------------------------------------------------
    bool				__fastcall	SetExclusiveMode();

    //----------------------------------------------------------------------------------
	// Func:	SetExclusiveMode()
    // Desc:	This function will switch DirectDraw to normal or windowed mode
    // Returns:	Upon success, this function returns TRUE, otherwise it returns FALSE
    //----------------------------------------------------------------------------------
    bool				__fastcall	SetNormalMode();

    //----------------------------------------------------------------------------------
    // Prop:	DirectDraw - returns a pointer to the DirectDraw object used in this unit.
    // 			PrimarySurface - returns a pointer to the primary surface.
	// 			BackBuffer - returns a pointer to the back buffer.
    // Notes:	Use these properties to access the actual DirectDraw objects in the
    //			case that you want to perform more advanced DirectDraw functions.
    // Result:	If the result is NULL, the object has not been created or for some
    //			other reason does not exist.
    //----------------------------------------------------------------------------------
	__property	LPDIRECTDRAW			DirectDraw      = {read=fDirectDraw};
    __property	LPDIRECTDRAWSURFACE		PrimarySurface  = {read=fPrimarySurface};
    __property	LPDIRECTDRAWSURFACE		BackBuffer      = {read=fBackBuffer};

    //----------------------------------------------------------------------------------
    // Prop:	Width - the current display width (if application is maximized)
    //			Height - the current display height (if application is maximized)
    //			BitsPerPixel - the current display bits per pixel (if application is maximized)
    //			BackBuffers - the number of DirectDraw backbuffers allocated.
    //----------------------------------------------------------------------------------
    __property	int					Width           = {read=fWidth};
    __property	int					Height          = {read=fHeight};
    __property	int					BitsPerPixel    = {read=fBitsPerPixel};
    __property  int                 BackBuffers     = {read=fBackBufferCount,write=SetBackBuffers,default=1};

    //----------------------------------------------------------------------------------
    // Prop:	TransparentColor - please note that this is not working in version 1.0
    //----------------------------------------------------------------------------------
//    __property  TColor              TransparentColor = {read=fTransparentColor,write=fTransparentColor,default=clMaroon};

    //----------------------------------------------------------------------------------
    // internal vars, methods, etc.
    //----------------------------------------------------------------------------------
						__fastcall 	TDirectDraw(TComponent* Owner);
    virtual				__fastcall 	~TDirectDraw();

    //----------------------------------------------------------------------------------
	// Func:	Debug(), Error(), ErrorMessage()
    // Desc:	This function can be used to "Log" an message for later retrieval using
    // 			the ErrorStrings property.
    // Params:	msgclass - one of DBG_IN, DBG_OUT, DBG_INFO, DBG_NOTE, DBG_ERROR
    //			functiondesc - generally meant to be a function reference.
    //			msg - the message to display
    //			errorvalue - the result from calling a DirectDraw function.
    // Example:	void MyFunc()
    //			{
    //			DirectDraw->Debug( DBG_IN, Name+"::MyFunc()", "Entering func");
    //			.
    //			.
    //				DirectDraw->Debug( DBG_INFO, Name+"::MyFunc()", "Middle of func");
    //			.
    //			.
    //          DirectDraw->Debug( DBG_OUT, Name+"::MyFunc()", "Exitting func");
    //          }
    // Note:	These functions require that DirectDraw->Debugging == true;
    //----------------------------------------------------------------------------------
    void				__fastcall	Debug( int msgclass, System::AnsiString functiondesc, System::AnsiString msg );
    void				__fastcall	Error( int errorvalue, System::AnsiString functiondesc, System::AnsiString errormsg );
    System::AnsiString	__fastcall	ErrorMessage(int errorvalue);

    //----------------------------------------------------------------------------------
	// Prop:	Debugging	 - if FALSE, the Debug() function does not work.
    // Prop:	ErrorValue 	 - the last result from any DirectDraw function call
    // Prop:	ErrorCount 	 - the number of messages that have been "logged" since
    //							startup or since setting DirectDraw->ErrorCount = n;
    // Prop:	ErrorStrings - a list of all messages logged since startup or since
    //							DirectDraw->ErrorStrings->Clear();
    //----------------------------------------------------------------------------------
	__property 	bool				Debugging       = {read=fDebugging,write=fDebugging,default=false};
    __property	HRESULT				ErrorValue      = {read=fErrorValue};
    __property	int					ErrorCount      = {read=fErrorCount,write=fErrorCount};
    __property	TStringList*		ErrorStrings    = {read=fErrorStrings};

private:

    //----------------------------------------------------------------------------------
    // internal vars, methods, etc. --- just ignore the rest of these declarations... 
    //----------------------------------------------------------------------------------

    HICON							fMouseCursor;
    HBITMAP							fMouseUnder;
    Graphics::TBitmap*				fMouseUnderCursor;
    Graphics::TBitmap*				fMouseBitmap;
    POINT							fMousePos;
    int								fMouseWidth;
    int								fMouseHeight;
    bool							fMouseHasMoved;
    bool							fMouseFirstTimeDrawn;
    TControl*						fMouseBusy;
    TRect							fMouseNewRect;
    TRect							fMouseOldRect;

    TCanvas*                        fPrimaryCanvas;
    TCanvas*                        fBackCanvas;

    TColor                          fTransparentColor;
    bool							fDrawingOnPrimarySurface;
    bool							fDrawingOnBackBuffer;
    bool							fAutoFlip;
    TRect							fAutoFlipClipRect;

    bool							fDebugging;
    HRESULT							fErrorValue;
    int								fErrorCount;
    TStringList*					fErrorStrings;
    int								fErrorIndent;

	LPDIRECTDRAW					fDirectDraw;
    LPDIRECTDRAWSURFACE				fPrimarySurface;
    LPDIRECTDRAWSURFACE				fBackBuffer;
    LPDIRECTDRAWPALETTE				fPalette;

    HDC								fBackBufferDC;
    unsigned long					fBackBufferDCCnt;
    HDC								fPrimarySurfaceDC;
    unsigned long					fPrimarySurfaceDCCnt;
    HDC								fCanvasDC;

    int								fWidth;
    int								fHeight;
    int								fBitsPerPixel;
    int								fBackBufferCount;
    bool							fExclusiveModeSet;

    bool				__fastcall	CreateDirectDraw();
    bool				__fastcall	ReleaseDirectDraw();
	bool				__fastcall	CreateSurfaces();
    bool				__fastcall	ReleaseSurfaces();
    void		        __fastcall	SetBackBuffers( int numbackbuffers );
    HDC			        __fastcall	GetBackBufferDC();
    HDC			        __fastcall	GetPrimarySurfaceDC();
    HDC			        __fastcall	ReleaseBackBufferDC();
    HDC			        __fastcall	ReleasePrimarySurfaceDC();
    
    void				__fastcall	SetMouseCursor( HICON cursor );
    void				__fastcall	SetMousePos( int x,int y );
    void				__fastcall	UpdateMouseCursor(bool forced=false);
    void				__fastcall	DebugMouseCursor();
    void				__fastcall	PaintMouseCursor();
    void				__fastcall	UnpaintMouseCursor();

    __property			TControl*	MouseBusy={read=fMouseBusy,write=fMouseBusy,default=false};
};
//---------------------------------------------------------------------------
// Various error levels used by the Debug() function. See above for more info.
//---------------------------------------------------------------------------
#define	DBG_IN		0	// -- entering the function, add indent
#define	DBG_OUT		1	// -- exiting a function, remove indent
#define	DBG_INFO	2	// -- 'Info'
#define	DBG_NOTE	3	// -- 'Note'
#define	DBG_ERROR	4	// -- 'Error'
//---------------------------------------------------------------------------
extern  TDirectDraw*	DirectDraw;
//---------------------------------------------------------------------------
#endif


